// Copyright (c) 2013, Christopher Best
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
// http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

part of realtime_data_model;

/// A class to create local documents with no persistence
class LocalDocumentProvider extends DocumentProvider {
  Document get document => _document;
  Document _document;
  String _initData;

  /// Create a local [Document] which is provided to the returned [Future]
  /// initializeModel is called before the future completes
  Future<Document> loadDocument([initializeModel(Model)]) {
    var model = new _LocalModel();
    // create a document with the model
    var document = new _LocalDocument(model);
    // initialize with data if provided
    if(_initData != null) {
      model._initializeFromJson(_initData);
    } else {
      model._initialize(initializeModel);
    }
    _document = document;
    var completer = new Completer();
    // complete with document
    completer.complete(document);
    return completer.future;
  }

  Future<Map> exportDocument() {
    return new Future.value((_document.model as _LocalModel)._export());
  }

  static bool _isCustomObject(dynamic object) {
    return object is CustomObject && object._isLocalCustomObject;
  }

  LocalDocumentProvider([String data]) {
    _initData = data;
  }
}